# $Id: namenormalize.py 4072 2010-11-29 09:07:27Z place $
# $URL: https://svn.non-gnu.uvt.nl/uvt-dev/tags/sources/meresco/bzv-9.9.6/src/delfttilburg/normalize/namenormalize.py $
## begin license ##
#
#    "Delft-Tilburg" (delfttilburg) is a package containing shared code
#    between the Delft "Discover" and Tilburg "Beter Zoeken & Vinden" projects.
#    Both projects are based on Meresco Software (http://meresco.com)
#    Copyright (C) 2008-2009 Technische Universiteit Delft http://www.tudelft.nl
#    Copyright (C) 2008-2009 Universiteit van Tilburg http://www.uvt.nl
#    Copyright (C) 2008-2009 Seek You Too (CQ2) http://www.cq2.nl
#
#    This file is part of "Delft-Tilburg".
#
#    "Delft-Tilburg" is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    "Delft-Tilburg" is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with "Delft-Tilburg"; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
## end license ##
import pprint

from re import compile, UNICODE, IGNORECASE
from unicodedata import normalize as uNormalize
normalForm = "NFKC"

def lower(initials):
    return initials.lower()

wordSplitter = compile(r'(?<=\w)\w*[ \-\.]*', UNICODE)

def getInitials(firstNames):
    return [aChar.lower() for aChar in wordSplitter.split(firstNames) if aChar]

def addDots(initiialList):
    return ".".join(initiialList) + "."

def getDoubleInitials(firstNames):
    return [initial.lower() for initial in firstNames.split('.') if initial]

global compiledExpressions
global compiledSpecificExpressions
global prefixArray

prefixArray = [
    'an',
    'and',
    'as',
    'de',
    'dei',
    'den',
    'der',
    'des',
    'di',
    'du',
    'for',
    'from',
    'les',
    'of',
    'on',
    '\'s',
    '\'t',
    'the',
    'und',
    'van',
    'von',
    'voor',
    ]

def buildRegexps():
    global prefixArray
    prefixexpr = "".join([el + "|" for el in prefixArray ])
    prefixes = "(?:" + prefixexpr + ")(?:\s+(?:" + prefixexpr + ")){0,2}"

    orprefix = r"(?P<prefix>|" + prefixes + ")"
    prefix   = r"(?P<prefix>"  + prefixes + ")"

    lastname      = r"\(?(?P<lastname>[\w\-\'\s]+)\)?"

    punctuation = r'(,\s*|,)'
    orcomment= r'(?:|\([\w\s\.\-]+\))\s*'

    expressions = [
        #self.assertLastnameAndFirstInitial("O'N. Fisher E.", "Eric O' N. Fisher")
        (r'^(?P<initials>\w)\w+\s*(?P<lastname>O\'.+)', getInitials),

        # e.g. "M.P.D."  (WhaAAtevaesdf)
        ( r'(?P<initials>(\w\. )+)'+lastname, getInitials),
        # e.g. "Johnson-Peter'son, M. P-  D (Some-Extra stuff.) van den"
        ('^' + lastname + punctuation + r'(?P<initials>(?:\w[\.\-]{1,2}\s*)+)\s*'+ orcomment + orprefix+'$',getInitials),

        # e.g. "Dijk, van, J.K.,"
        #    ('^' + lastname + punctuation + prefix +  '\,*' + r'(?P<initials>' + initials + ')\,*' + '$',getInitials),
        ('^' + lastname + punctuation + prefix + punctuation + r'(?P<initials>([A-Z]\.?)+)',getInitials),

        # e.g. "Johnson, MPD"
        ('^'+ lastname + punctuation + r'(?P<initials>[A-Z]+)$', lower),

        # UvT toegevoegd
        # e.g. "Johnson, M,"
        ('^' + lastname + punctuation + r'(?P<initials>[A-Z]+),?$', lower),
        # e.g. "Johnson, M," or e.g. "Johnson, M.," or e.g. "Johnson, M.A,"
        ('^' + lastname + punctuation + r'(?P<initials>([A-Z]\.?)+),?$', lower),
        # e.g. "Johnson, John,"
        ('^' + lastname + punctuation+r'(?P<initials>\w+\,)$', lower),

        # e.g. "Johnson, M P-D"
        ('^' + lastname + punctuation + r'(?P<initials>[\w\s\-]+)$', getInitials),
        # e.g. "Johnson, Pete  D.L.  "
        ('^' + lastname + punctuation + r'(?P<initials>\w+\s+(\w\.)+)\s*' + orprefix+'$',getInitials),
        # e.g. "Johnson, L. Pete"
        ('^' + lastname + punctuation + r'(?P<initials>(\w\.)+\s+\w+)\s*' +'$',getInitials),
        # e.g. "Johnson,  M. Ph  D."
        ('^' + lastname + punctuation + r'(?P<initials>(\w{1,2}[\.\s])+)\s*' + orprefix + '$', getDoubleInitials),
        # e.g. "Johnson, P.Desiree"
        ('^' + lastname + punctuation + r'(?P<initials>(\w\.\w+))$', getInitials),
        # e.g. "Johnson, John-Peter George (Lord-of-the-Rings)"
        ('^' + lastname + punctuation + r'(?P<initials>[\w\s\-]+)'+r'\s*(?:\([\w\s\.\-]+\))\s*', getInitials),
        # e.g. "E.J. Groeneveld (Lord)"
        (r'^(?P<initials>(\w+[\s\.]{1,2})+)' + lastname + r'\s*(?:\([\w\s\.\-]+\))\s*', getInitials),
        # e.g. "Maria P.Johnson", "Maria.P.Johnson.", "Javier Hidalgo", "Peter M Robinson", "Raymond H.J.M Gradus"
        (r'^(?P<initials>(\w+[\s\.]{1,2})+)' + lastname, getInitials),
        # e.g. "Stephanie van der Geest", "Matthijs de Jong"

        # e.g. "Maria J."

        # e.g. "(Johnson)", "Johnson-Peter'son?"
        (r'^\(?(?P<lastname>[\w\-\']+)\??\)?$', None),
        # e.g. "Association for Computing Machinery, Inc."
        (r'^(?P<lastname>[\w\-]+(\s[\w\-]+)*)((, )(Inc|Inc\.|Incorporated))?$', None),

        # "Adeline A."
        #(r"(?P<lastname>\w+)\s\w\.", None),
        # "Agricola, Georgius."
        #(r"(?P<lastname>\w+),\s(?P<initials>\w)\w*?\.", getInitials),
        ]

    specificExpressions = [
        # (Lord), (Lady), etc
        (IGNORECASE, r'\(Captain\)|\(Baron\)|\(Countess\)|\(Cid\)|\(Earl\)|\(Fellow\)|\(Fils\)|\(Frhr\)|\(Freiherr\)|\(Graaf\)|\(Graf\)|\(Grossherzog\)|\(Jeune\)|\(Jr\)|\(Lady\)|\(Lord\)|\(Meenenaer\)|\(Prins\)|\(Prinses\)|\(Ritter\)|\(Sir\)|\(Wzn\)|\(Daventriensis\)|\(\)|\(\)|\(\)|\(\)|\(\)'),
        # "Roman numbers optionally in ( ), "XVI", "(MXXII)"
        (0, r'^\(?[IVXMCL]+\)?$'),
        # "1e Divisie", "1998", "1997, Munich" etc
        (IGNORECASE, r'^\d+(th|nd|st|e)?,?\s?(\w+)?$'),
        # Academy, University, Sciene etc
        (IGNORECASE, r'(ad\shoc)?[\w\.]*?(\d\d\d\d)?\s?(?P<result>[\w\s\-]*?(a[ck]adem|science|abteilung|society|museum|school|universit|college|foundation|corporation|company|ass?ocia[tcz]ion|organi[sz]ati|observatory|committee|limited|ltd|division|afdeling|research|laborator|centre|center|centrum|command|all?gemei?ne|international|maatschappij|bibliothe|library|departe?ment|group|service|[ck]ommissi|part(y|ij)|stichting|panel|project|council|gemeinschaft|arbeitsgruppe|werkgroep|arbeitskreis|gemeenschap|verein|forschung|institut|experiment|hospital|agency|assembly|atelier|forum|board|gmbh|fabrie?k|congres)[\w\s\-]*)'),
        # "'Het Gesticht', Een instelling voor programmeurs."
        (0, r"\'(?P<result>[\w\s]+)\',[\s\w]+"),
        ]

    global compiledExpressions
    global compiledSpecificExpressions

    compiledExpressions = [(compile(expression, UNICODE),f) for expression,f in expressions]
    compiledSpecificExpressions = [compile(expression, flags|UNICODE) for flags, expression in specificExpressions]



# N.B.
# Call the above function at least once at initialisation
buildRegexps()

def _breakUp(name):
    if type(name) != unicode:
        name = unicode(name)
    name = uNormalize(normalForm, name)
#
#    for expression, initialsPostProcess in compiledExpressions:
#        m = expression.match(name)
#        if m:
#            for key  in m.groupdict():
#                print "key: \"" + key
#                print "\" value: \"" + m.groupdict()[key] + "\""
#
    for expression in compiledSpecificExpressions:
        m = expression.match(name)
        if m:
            if 'result' in m.groupdict():
                return m.groupdict()['result'].lower(), '', ''
            else:
                return None

    for expression, initialsPostProcess in compiledExpressions:
        m = expression.match(name)
        if m:
            lastname = m.groupdict()['lastname'].lower()
            lastname = lastname.strip()
            if initialsPostProcess:
                initials = addDots(initialsPostProcess(m.groupdict()['initials']))
            else:
                initials = ''
            prefix = m.groupdict().get('prefix', "")
#            print name + " ==lastname=>" + str(lastname) +  '==initials=' + str(initials) + '==prefix=' + str(prefix) + '==='

            # remove whitespace in "O' Neal"
            lastname = str(lastname)
            apospace = compile("o\'\s")
            lastname = apospace.sub("o'",lastname)
            return lastname, str(initials), str(prefix)
    return None


def dots(s):
    return "".join([c + "." for c in s])

def _firstChar(s):
    return s and s[0]

_capitalizeRegexp = compile(r"(ij|'s|\w)([\w']*)", UNICODE)
def _capitalizeSubstite(match):
    firstLetter = match.group(1)
    tail = match.group(2)
    word = firstLetter+tail
    pp = pprint.PrettyPrinter(indent=2)

    #  O'Donnel instead of O'donnel
    if tail and tail[0] == "'":
        upchar = ''
        rest = ''
        if len(tail) >= 2:
            upchar =  tail[1].upper()
        if len(tail) >= 3:
            rest = tail[2:]
        tail = "'" + upchar + rest


    # FIXME
    if word in ['van', 'der','voor',  'for', 'and', 'of', 'from', 'on', 'the', 'an', 'as', "'s", 'und', 'von', 'de', 'des', 'les', 'du', 'dei', 'di']:
        return word
    return firstLetter.upper() + tail

def _capitalize(aString):
    return str(_capitalizeRegexp.sub(_capitalizeSubstite, unicode(aString)))

def _helsing_v((lastname, initials, prefix)):
    prefix = ''.join([aChar == "'t" and aChar or aChar+"." for aChar in getInitials(prefix)])
    result = _capitalize(lastname)
    if prefix:
        return result + ", " + prefix
    return result

def _helsing_a_v((lastname, initials, prefix)):

    prefix = ''.join([aChar == "'t" and aChar or aChar+"." for aChar in getInitials(prefix)])
    initials = initials.split('.', 1)[0] + '.'
    result = "%s, %s" % (_capitalize(lastname), _capitalize(initials))
    if prefix:
        return result + " " + prefix
    return result

def _helsing_ab_van((lastname, initials, prefix)):
    if initials:
        result = "%s, %s" % (_capitalize(lastname), _capitalize(initials))
    else:
        result = _capitalize(lastname)
    if prefix:
        return result + " " + prefix
    return result

def lastname(unparsedName):
    parts = _breakUp(unparsedName)
    if parts:
        yield _helsing_v(parts)

def lastnameAndFirstInitial(unparsedName):
    parts = _breakUp(unparsedName)
    if parts:
        yield _helsing_a_v(parts)

def lastnameAndInitials(unparsedName):
    parts = _breakUp(unparsedName)
    if parts:
        yield _helsing_ab_van(parts)

def firstLetter(unparsedName):
    parts = _breakUp(unparsedName)
    if parts:
        lastname, initials, prefix = parts
        lastname = unicode(lastname)
        yield str(lastname[:1].upper())

def unparsable(unparsedName):
    parts = _breakUp(unparsedName)
    if not parts:
        yield unparsedName

def overridePrefixList(lst):
    global prefixArray
    prefixArray = lst
    buildRegexps()
