from lxml.etree import parse, tostring
from meresco.core import Observable
from urllib import urlopen
from urlparse import urlsplit
import re

class LoanInfo(Observable):
    def __init__(self, infra = None):
        Observable.__init__(self)
        self.infra = infra
        self.serverConfig  = self.infra.configRoot
        self.baseurl = self.serverConfig.get('baseurl', section='system')
        self.nsMap = self.serverConfig.getSectionDict('namespaces')
        collectionString =  self.serverConfig.get('collectionList', section="collections")
        self.collections = []
        if collectionString: self.collections = collectionString.split("\n")
        exceptionString =  self.serverConfig.get('exceptionList', section="collections")
        self.exceptions = []
        if exceptionString: self.exceptions = exceptionString.split("\n")

    def okText(self):
       CRLF = "\r\n"
       ContentType = "Content-Type: text/html"
       okText = "HTTP/1.0 200 OK" + CRLF + \
                ContentType + CRLF + \
                CRLF
       return okText

    def handleRequest(self, RequestURI='', **kwargs):
        Scheme, Netloc, Path, Query, Fragment = urlsplit(RequestURI)
        ppn = Path.split("/loaninfo/")[1]
        yield self.okText()
        locations, genre = self.getEPNs(ppn)
        #print locations
        #print genre
        epn = ""
        for location in locations:
            if location["epn"]:
                epn = location["epn"]
                break
        url = "http://search.uvt.nl/locator1/Locator?epn=%(epn)s&sid=uvt-opc-uvt.nl&genre=%(genre)s" %{'epn': epn, 'genre': genre}
        response = parse(urlopen(url))
        #print tostring(response)
        yield """<html>
 <head>
  <title>LoanInfo for %s</title>
 <head>
 <body>
  <table width="100%%" border="1">
   <thead>
    <tr>
     <th>Location</th><th>Status</th><th>Shelfmark</th><th>Notes</th>
    </tr>
   </thead>
   <tbody>"""  %ppn
        for location in locations:
            epn    = location["epn"]
            sgn    = location["location"]
            action = location["action"]
            cdrom  = location["cdrom"]
            ulc    = location["ulc"]
            nsMap = {'dlf': "http://diglib.org/ilsdi/1.1"}
            path = "/records/record[epn='%s']/availability/dlf:collection/dlf:record/dlf:simpleavailability" %epn
            try:
                avail = response.xpath(path, nsMap)[0]
            except: pass
            try:
                availability = avail.xpath("dlf:availabilitystatus/text()", nsMap)[0]
            except: availability = ""
            #print availability
            try:
                msg = avail.xpath("dlf:availabilitymsg/text()", nsMap)[0]
            except: msg = ""
            if genre == "journal":
                path = "/records/record[epn='%s']/action/text()" %location["epn"]
                action = response.xpath(path, nsMap)[0]
            status, msg = self.getStatus(genre, availability, action, msg, ulc, cdrom)
            if genre == "journal":
                path = "/records/record[epn='%s']/collection/text()" %location["epn"]
                collection = response.xpath(path, nsMap)[0]
                msg = location["holding"] + " (" + collection + ") - " + msg

            yield """    <tr>
     <td>Library Tilburg University</td>
     <td>%(status)s</td>
     <td>%(sgn)s</td>
     <td>%(msg)s</td>
    </tr>""" %{'status': status, 'sgn': sgn, 'msg': msg}
        yield """   </tbody>
  </table>
  <br/>
  <iframe src="http://search.uvt.nl/id/opc-uvt-nl:%s" width="1000p" height="1000p"/>
 </body>
</html>""" %ppn

    def getEPNs(self, ppn):
        recordId = "opc-uvt-nl:" + ppn
        mods = parse(self.any.getStream(recordId, 'mods'))
        #print tostring(mods)
        genre = mods.xpath("/mods:mods/mods:genre/text()", self.nsMap)[0]
        if genre.startswith("info:eu-repo/semantics/"):
            genre = genre.split("info:eu-repo/semantics/")[1]
        copies = mods.xpath("/mods:mods/mods:location/mods:holdingSimple/mods:copyInformation", self.nsMap)
        locations = []
        for copyInformation in copies:
            blok = {}
            EPN = copyInformation.xpath("mods:note[@type='epn']/text()", self.nsMap)[0]
            if EPN:
                blok["epn"] = EPN
                location = copyInformation.xpath("mods:shelfLocator/text()", self.nsMap)
                location = location and location[0] or ''
                if location:
                    blok["location"] = location
                    action = "stack"
                    if self.inDepot(location): action = "depot"
                    blok["action"] = action
                    try:
                        ulc = copyInformation.xpath("mods:note[@type='status']/text()", self.nsMap)[0]
                    except: ulc = ""
                    blok["ulc"] = ulc
                    notes = copyInformation.xpath("mods:note/text()", self.nsMap)
                    cdrom = ""
                    for note in notes:
                        expr = re.compile('CD[ -]ROM|DVD', re.IGNORECASE)
                        if expr.match(note): cdrom = "cdrom"
                    blok["cdrom"] = cdrom
                    if genre == "journal":
                        try:
                            holding = copyInformation.xpath("mods:enumerationAndChronology[@unitType='1']/text()", self.nsMap)[0]
                        except: holding = ""
                        blok["holding"] = holding
                    locations.append(blok)
        return locations, genre

    def getStatus(self, genre, availability, action, msg, ulc, cdrom):
        availabilitymsg =msg
        if ulc == "b": msg = "short loan"
        elif ulc == "f": msg = "not for loan"
        if genre == "series": return "See items in series", msg
        if genre == "journal":
            if action != "depot": return "In the library", msg
            if action == "depot": return "Request item from depot", msg
        if availability == "available" or availability == "possibly available" or availability == "unknown":
            if cdrom == "cdrom": return "Request at library desk", msg
            if action != "depot":
                if availability == "possibly available": msg = "not for loan"
                return "In the library", msg
            if action == "depot":
                return "Request item from depot", msg
        if availability == "not available":
           if "aangevraagd" in availabilitymsg or "Uitgeleend" in availabilitymsg:
               if ulc == "f": return "Requested by other user. Reserve", msg
               else: return "On loan. Reserve", msg
           if "Niet beschikbaar" in msg or "Niet uitleenbaar" in msg: return "Not available", msg
        return "Problem. Please ask support desk", msg

    def inDepot(self, location):
        for exception in self.exceptions:
            if re.match(exception+'\s',location):
                return False
        for collection in self.collections:
            if re.match(collection+'\s',location):
                return True
        return False
