# $Id: observablewebquery.py 4377 2011-06-13 22:23:52Z place $
# $URL: https://svn.non-gnu.uvt.nl/uvt-dev/tags/sources/meresco/bzv-9.9.6/src/observablewebquery.py $

import re
from meresco.core import Observable
from meresco.components.web import WebQuery
from cqlparser import cql2string

def addAndExactArguments(query, arguments):
	for fieldAndTerm in arguments:
		field, term = fieldAndTerm.split(':', 1)
		query.addFilter(field, term)

def webQueryParseTree(httpQuery, drilldownArguments, extra):
    cql = WebQuery(httpQuery, antiUnaryClause='meresco.exists exact true')
    cql = cql.replaceTerm('author', 'mods.name.namePart')
    cql = cql.replaceTerm('title', 'mods.titleInfo.title')
    cql = cql.replaceTerm('sgn', 'mods.location.holdingSimple.copyInformation.shelfLocator')
    if drilldownArguments:
        addAndExactArguments(cql, drilldownArguments)

    if extra:
        addAndExactArguments(cql, extra)

    return cql.ast

emptyQueryMatcher = re.compile('^\s*$')
punctuationCharacters = re.compile('[!:;,.&?+-] | [!:;,.&?]')
# note that ., + and - have special meanings
# norm.genre=...., +term, -term

class ObservableWebQuery(Observable):
    def parseQuery(self, httpQuery, drilldownArguments, extra=[]):
        if emptyQueryMatcher.match(httpQuery):
            return None, None
        httpQuery = punctuationCharacters.sub(" ", httpQuery)
        ast = webQueryParseTree(httpQuery, drilldownArguments, extra)
        #print ast.prettyPrint()
        return ast, cql2string(ast)
